@@ -15,11 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
 
 import pygame
@@ -115,6 +110,11 @@ class Game(object):
        specified in :data:`sge.image_directories` to use as the window
        icon.  If set to :const:`None`, the SGE chooses the icon.
 
+    .. attribute:: registered_classes
+
+       A list containing all classes which have been registered with
+       :meth:`sge.Game.register_class`.  (Read-only)
+
     .. attribute:: sprites
 
        A dictionary containing all loaded sprites, indexed by the
@@ -303,6 +303,7 @@ class Game(object):
         self.window_text = window_text
         self.window_icon = window_icon
 
+        self.registered_classes = []
         self.sprites = {}
         self.background_layers = {}
         self.backgrounds = {}
@@ -322,20 +323,25 @@ class Game(object):
         self._running = False
         self._clock = pygame.time.Clock()
         self._joysticks = []
+        self._js_names = {}
+        self._js_ids = {}
         self._pygame_sprites = pygame.sprite.LayeredDirty()
         self.mouse = sge.Mouse()
 
         # Setup sound channels
         self._available_channels = []
         if pygame.mixer.get_init():
-            for i in xrange(pygame.mixer.get_num_channels()):
+            for i in range(pygame.mixer.get_num_channels()):
                 self._available_channels.append(pygame.mixer.Channel(i))
 
         # Setup joysticks
         if pygame.joystick.get_init():
-            for i in xrange(pygame.joystick.get_count()):
+            for i in range(pygame.joystick.get_count()):
                 joy = pygame.joystick.Joystick(i)
                 joy.init()
+                n = joy.get_name()
+                self._js_names[i] = n
+                self._js_ids[n] = i
                 self._joysticks.append(joy)
 
         if not pygame.font.get_init():
@@ -412,8 +418,11 @@ class Game(object):
                 self._object_start_collision_ellipse[obj.id] = obj.collision_ellipse
                 self._object_start_collision_precise[obj.id] = obj.collision_precise
 
+            display_surface = self._window.copy()
+
             self.rooms[0].start()
             background = None
+            mouse_previous_rect = pygame.Rect(0, 0, 1, 1)
             numviews = 0
             _fps_time = 0
             self._clock.tick()
@@ -469,65 +478,70 @@ class Game(object):
                             else:
                                 obj.event_inactive_mouse_button_release(b)
                     elif event.type == pygame.JOYAXISMOTION:
-                        self.event_joystick_axis_move(event.joy, event.axis,
-                                                      event.value)
+                        jsname = self._js_names[event.joy]
+                        self.event_joystick_axis_move(jsname, event.joy,
+                                                      event.axis, event.value)
                         self.current_room.event_joystick_axis_move(
-                            event.joy, event.axis, event.value)
+                            jsname, event.joy, event.axis, event.value)
                         for obj in self.current_room.objects:
                             if obj.active:
                                 obj.event_joystick_axis_move(
-                                    event.joy, event.axis, event.value)
+                                    jsname, event.joy, event.axis, event.value)
                             else:
                                 obj.event_inactive_joystick_axis_move(
-                                    event.joy, event.axis, event.value)
+                                    jsname, event.joy, event.axis, event.value)
                     elif event.type == pygame.JOYHATMOTION:
-                        self.event_joystick_hat_move(event.joy, event.hat,
-                                                     *event.value)
+                        jsname = self._js_names[event.joy]
+                        self.event_joystick_hat_move(jsname, event.joy,
+                                                     event.hat, *event.value)
                         self.current_room.event_joystick_hat_move(
-                            event.joy, event.hat, *event.value)
+                            jsname, event.joy, event.hat, *event.value)
                         for obj in self.current_room.objects:
                             if obj.active:
                                 obj.event_joystick_hat_move(
-                                    event.joy, event.hat, *event.value)
+                                    jsname, event.joy, event.hat, *event.value)
                             else:
                                 obj.event_inactive_joystick_hat_move(
-                                    event.joy, event.hat, *event.value)
+                                    jsname, event.joy, event.hat, *event.value)
                     elif event.type == pygame.JOYBALLMOTION:
+                        jsname = self._js_names[event.joy]
                         self.event_joystick_trackball_move(
-                            event.joy, event.ball, *event.rel)
+                            jsname, event.joy, event.ball, *event.rel)
                         self.current_room.event_joystick_trackball_move(
-                            event.joy, event.ball, *event.rel)
+                            jsname, event.joy, event.ball, *event.rel)
                         for obj in self.current_room.objects:
                             if obj.active:
                                 obj.event_joystick_trackball_move(
-                                    event.joy, event.ball, *event.rel)
+                                    jsname, event.joy, event.ball, *event.rel)
                             else:
                                 obj.event_inactive_joystick_trackball_move(
-                                    event.joy, event.ball, *event.rel)
+                                    jsname, event.joy, event.ball, *event.rel)
                     elif event.type == pygame.JOYBUTTONDOWN:
-                        self.event_joystick_button_press(event.joy,
+                        jsname = self._js_names[event.joy]
+                        self.event_joystick_button_press(jsname, event.joy,
                                                          event.button)
                         self.current_room.event_joystick_button_press(
-                            event.joy, event.button)
+                            jsname, event.joy, event.button)
                         for obj in self.current_room.objects:
                             if obj.active:
-                                obj.event_joystick_button_press(event.joy,
-                                                                event.button)
+                                obj.event_joystick_button_press(
+                                    jsname, event.joy, event.button)
                             else:
                                 obj.event_inactive_joystick_button_press(
-                                    event.joy, event.button)
+                                    jsname, event.joy, event.button)
                     elif event.type == pygame.JOYBUTTONUP:
-                        self.event_joystick_button_release(event.joy,
+                        jsname = self._js_names[event.joy]
+                        self.event_joystick_button_release(jsname, event.joy,
                                                            event.button)
                         self.current_room.event_joystick_button_release(
-                            event.joy, event.button)
+                            jsname, event.joy, event.button)
                         for obj in self.current_room.objects:
                             if obj.active:
-                                obj.event_joystick_button_release(event.joy,
-                                                                  event.button)
+                                obj.event_joystick_button_release(
+                                    jsname, event.joy, event.button)
                             else:
                                 obj.event_inactive_joystick_button_release(
-                                    event.joy, event.button)
+                                    jsname, event.joy, event.button)
                     elif event.type == pygame.ACTIVEEVENT:
                         if event.gain:
                             if 2 & event.state:
@@ -563,9 +577,11 @@ class Game(object):
                     elif event.type == pygame.VIDEORESIZE:
                         if sge.DEBUG:
                             print('Video resize detected.')
+                        self._window.blit(display_surface, (0, 0))
                         self._window_width = event.w
                         self._window_height = event.h
                         self._set_mode()
+                        display_surface = self._window.copy()
                         self._background_changed = True
 
                 real_time_passed = self._clock.tick(self.fps)
@@ -595,6 +611,7 @@ class Game(object):
                 # Update objects (including mouse)
                 for obj in self.current_room.objects:
                     if obj.active:
+                        obj.event_begin_step(real_time_passed)
                         obj._update(time_passed, delta_mult)
                         obj.event_step(real_time_passed)
                     else:
@@ -606,6 +623,13 @@ class Game(object):
                     if obj is not None:
                         obj._detect_collisions()
 
+                # End step event
+                for obj in self.current_room.objects:
+                    if obj.active:
+                        obj.event_end_step(real_time_passed)
+                    else:
+                        obj.event_inactive_end_step(real_time_passed)
+
                 # Music control
                 if self._music is not None:
                     if pygame.mixer.music.get_busy():
@@ -632,27 +656,34 @@ class Game(object):
                     numviews = len(self.current_room.views)
                     self._background_changed = True
 
+                # Set xprevious and yprevious
+                for obj in self.current_room.objects:
+                    obj.xprevious = obj.x
+                    obj.yprevious = obj.y
+
                 # Redraw
                 if self._background_changed or background is None:
-                    w = max(1, self._window.get_width())
-                    h = max(1, self._window.get_height())
+                    w = max(1, display_surface.get_width())
+                    h = max(1, display_surface.get_height())
                     background = pygame.Surface((w, h))
                     b = self.current_room.background._get_background()
                     background.blit(b, (self._x, self._y))
-                    self._window.blit(background, (0, 0))
+                    display_surface.blit(background, (0, 0))
                     self._background_changed = False
-                    self._pygame_sprites.clear(self._window, background)
+                    self._pygame_sprites.clear(display_surface, background)
                     for sprite in self._pygame_sprites:
                         sprite.rect = pygame.Rect(0, 0, 1, 1)
                         sprite.image = pygame.Surface((1, 1))
                         sprite.image.set_colorkey((0, 0, 0))
                     self._pygame_sprites.update()
-                    self._pygame_sprites.draw(self._window)
-                    dirty = [self._window.get_rect()]
+                    self._pygame_sprites.draw(display_surface)
+                    dirty = [display_surface.get_rect()]
                 else:
-                    self._pygame_sprites.clear(self._window, background)
+                    self._pygame_sprites.clear(display_surface, background)
                     self._pygame_sprites.update()
-                    dirty = self._pygame_sprites.draw(self._window)
+                    dirty = self._pygame_sprites.draw(display_surface)
+
+                self._window.blit(display_surface, (0, 0))
 
                 top_bar = pygame.Rect(0, 0, w, self._y)
                 bottom_bar = pygame.Rect(0, h - self._y, w, self._y)
@@ -671,6 +702,22 @@ class Game(object):
                     self._window.fill((0, 0, 0), right_bar)
                     dirty.append(right_bar)
 
+                if (not self.grab_input and self.mouse.visible and
+                        self.mouse.sprite is not None):
+                    mx, my = pygame.mouse.get_pos()
+                    mouse_rect = pygame.Rect(
+                        mx - self.mouse.sprite.origin_x,
+                        my - self.mouse.sprite.origin_y,
+                        self.mouse.sprite.width, self.mouse.sprite.height)
+                    img = self.mouse.sprite._get_image(
+                        self.mouse.image_index, self.mouse.image_xscale,
+                        self.mouse.image_yscale, self.mouse.image_rotation,
+                        self.mouse.image_alpha, self.mouse.image_blend)
+                    self._window.blit(img, mouse_rect)
+                    dirty.append(mouse_rect)
+                    dirty.append(mouse_previous_rect)
+                    mouse_previous_rect = mouse_rect
+
                 if sge.hardware_rendering:
                     pygame.display.flip()
                 else:
@@ -764,45 +811,50 @@ class Game(object):
                     for obj in self.current_room.objects:
                         obj.event_paused_mouse_button_release(b)
                 elif event.type == pygame.JOYAXISMOTION:
-                    self.event_paused_joystick_axis_move(event.joy, event.axis,
-                                                         event.value)
+                    jsname = self._js_names[event.joy]
+                    self.event_paused_joystick_axis_move(
+                        jsname, event.joy, event.axis, event.value)
                     self.current_room.event_paused_joystick_axis_move(
-                        event.joy, event.axis, event.value)
+                        jsname, event.joy, event.axis, event.value)
                     for obj in self.current_room.objects:
                         obj.event_paused_joystick_axis_move(
-                            event.joy, event.axis, event.value)
+                            jsname, event.joy, event.axis, event.value)
                 elif event.type == pygame.JOYHATMOTION:
-                    self.event_paused_joystick_hat_move(event.joy, event.hat,
-                                                        *event.value)
+                    jsname = self._js_names[event.joy]
+                    self.event_paused_joystick_hat_move(
+                        jsname, event.joy, event.hat, *event.value)
                     self.current_room.event_paused_joystick_hat_move(
-                        event.joy, event.hat, *event.value)
+                        jsname, event.joy, event.hat, *event.value)
                     for obj in self.current_room.objects:
                         obj.event_paused_joystick_hat_move(
-                            event.joy, event.hat, *event.value)
+                            jsname, event.joy, event.hat, *event.value)
                 elif event.type == pygame.JOYBALLMOTION:
+                    jsname = self._js_names[event.joy]
                     self.event_paused_joystick_trackball_move(
-                        event.joy, event.ball, *event.rel)
+                        jsname, event.joy, event.ball, *event.rel)
                     self.current_room.event_paused_joystick_trackball_move(
-                        event.joy, event.ball, *event.rel)
+                        jsname, event.joy, event.ball, *event.rel)
                     for obj in self.current_room.objects:
                         obj.event_paused_joystick_trackball_move(
-                            event.joy, event.ball, *event.rel)
+                            jsname, event.joy, event.ball, *event.rel)
                 elif event.type == pygame.JOYBUTTONDOWN:
-                    self.event_paused_joystick_button_press(event.joy,
-                                                            event.button)
+                    jsname = self._js_names[event.joy]
+                    self.event_paused_joystick_button_press(
+                        jsname, event.joy, event.button)
                     self.current_room.event_paused_joystick_button_press(
-                        event.joy, event.button)
+                        jsname, event.joy, event.button)
                     for obj in self.current_room.objects:
-                        obj.event_paused_joystick_button_press(event.joy,
-                                                               event.button)
+                        obj.event_paused_joystick_button_press(
+                            jsname, event.joy, event.button)
                 elif event.type == pygame.JOYBUTTONUP:
-                    self.event_paused_joystick_button_release(event.joy,
-                                                              event.button)
+                    jsname = self._js_names[event.joy]
+                    self.event_paused_joystick_button_release(
+                        jsname, event.joy, event.button)
                     self.current_room.event_paused_joystick_button_release(
-                        event.joy, event.button)
+                        jsname, event.joy, event.button)
                     for obj in self.current_room.objects:
-                        obj.event_paused_joystick_button_release(event.joy,
-                                                                 event.button)
+                        obj.event_paused_joystick_button_release(
+                            jsname, event.joy, event.button)
                 elif event.type == pygame.ACTIVEEVENT:
                     if event.gain:
                         if 2 & event.state:
@@ -853,6 +905,19 @@ class Game(object):
             # Redraw
             self._window.blit(background, (0, 0))
 
+            if (not self.grab_input and self.mouse.visible and
+                    self.mouse.sprite is not None):
+                mx, my = pygame.mouse.get_pos()
+                mouse_rect = pygame.Rect(
+                    mx - self.mouse.sprite.origin_x,
+                    my - self.mouse.sprite.origin_y,
+                    self.mouse.sprite.width, self.mouse.sprite.height)
+                img = self.mouse.sprite._get_image(
+                    self.mouse.image_index, self.mouse.image_xscale,
+                    self.mouse.image_yscale, self.mouse.image_rotation,
+                    self.mouse.image_alpha, self.mouse.image_blend)
+                self._window.blit(img, mouse_rect)
+
             pygame.display.flip()
 
         # Restore the look of the screen from before it was paused
@@ -864,6 +929,23 @@ class Game(object):
         """Unpause the game."""
         self._paused = False
 
+    def register_class(self, cls):
+        """Register a class with the SGE.
+
+        Registered classes can be used to index objects by, e.g. for
+        :attr:`sge.Room.objects_by_class`.  A list of all currently
+        registered classes can be found in :attr:`registered_classes`.
+
+        """
+        if cls not in self.registered_classes:
+            self.registered_classes.append(cls)
+
+            for room in self.rooms:
+                room.objects_by_class[cls] = []
+                for obj in room.objects:
+                    if isinstance(obj, cls):
+                        room.objects_by_class[cls].append(obj)
+
     def event_game_start(self):
         """Game start event.
 
@@ -959,13 +1041,16 @@ class Game(object):
         """
         pass
 
-    def event_joystick_axis_move(self, joystick, axis, value):
+    def event_joystick_axis_move(self, name, ID, axis, value):
         """Joystick axis move event.
 
         Called when an axis on a joystick changes position.
 
         Arguments:
 
+        - ``name`` -- The name of the joystick.
+        - ``ID`` -- The number of the joystick, where ``0`` is the first
+          joystick.
         - ``value`` -- The tilt of the axis as a float from ``-1`` to
           ``1``, where ``0`` is centered, ``-1`` is all the way to the
           left or up, and ``1`` is all the way to the right or down.
@@ -976,7 +1061,7 @@ class Game(object):
         """
         pass
 
-    def event_joystick_hat_move(self, joystick, hat, x, y):
+    def event_joystick_hat_move(self, name, ID, hat, x, y):
         """Joystick HAT move event.
 
         Called when a HAT switch (also called the POV hat, POV switch,
@@ -984,6 +1069,9 @@ class Game(object):
 
         Arguments:
 
+        - ``name`` -- The name of the joystick.
+        - ``ID`` -- The number of the joystick, where ``0`` is the first
+          joystick.
         - ``x`` -- The horizontal position of the HAT, where ``0`` is
           centered, ``-1`` is left, and ``1`` is right.
         - ``y`` -- The vertical position of the HAT, where ``0`` is
@@ -995,15 +1083,16 @@ class Game(object):
         """
         pass
 
-    def event_joystick_trackball_move(self, joystick, ball, x, y):
+    def event_joystick_trackball_move(self, name, ID, ball, x, y):
         """Joystick trackball move event.
 
         Called when a trackball on a joystick moves.
 
         Arguments:
 
-        - ``joystick`` -- The number of the joystick, where ``0`` is the
-          first joystick.
+        - ``name`` -- The name of the joystick.
+        - ``ID`` -- The number of the joystick, where ``0`` is the first
+          joystick.
         - ``ball`` -- The number of the trackball, where ``0`` is the
           first trackball on the joystick.
         - ``x`` -- The horizontal relative movement of the trackball.
@@ -1012,22 +1101,34 @@ class Game(object):
         """
         pass
 
-    def event_joystick_button_press(self, joystick, button):
+    def event_joystick_button_press(self, name, ID, button):
         """Joystick button press event.
 
         Called when a joystick button is pressed.
 
+        Arguments:
+
+        - ``name`` -- The name of the joystick.
+        - ``ID`` -- The number of the joystick, where ``0`` is the first
+          joystick.
+
         See the documentation for
         :func:`sge.get_joystick_button_pressed` for more information.
 
         """
         pass
 
-    def event_joystick_button_release(self, joystick, button):
+    def event_joystick_button_release(self, name, ID, button):
         """Joystick button release event.
 
         Called when a joystick button is released.
 
+        Arguments:
+
+        - ``name`` -- The name of the joystick.
+        - ``ID`` -- The number of the joystick, where ``0`` is the first
+          joystick.
+
         See the documentation for
         :func:`sge.get_joystick_button_pressed` for more information.
 
@@ -1212,7 +1313,7 @@ class Game(object):
         """
         pass
 
-    def event_paused_joystick_axis_move(self, joystick, axis, value):
+    def event_paused_joystick_axis_move(self, name, ID, axis, value):
         """Joystick axis move event when paused.
 
         See the documentation for
@@ -1221,7 +1322,7 @@ class Game(object):
         """
         pass
 
-    def event_paused_joystick_hat_move(self, joystick, hat, x, y):
+    def event_paused_joystick_hat_move(self, name, ID, hat, x, y):
         """Joystick HAT move event when paused.
 
         See the documentation for
@@ -1230,7 +1331,7 @@ class Game(object):
         """
         pass
 
-    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
+    def event_paused_joystick_trackball_move(self, name, ID, ball, x, y):
         """Joystick trackball move event when paused.
 
         See the documentation for
@@ -1240,7 +1341,7 @@ class Game(object):
         """
         pass
 
-    def event_paused_joystick_button_press(self, joystick, button):
+    def event_paused_joystick_button_press(self, name, ID, button):
         """Joystick button press event when paused.
 
         See the documentation for
@@ -1250,7 +1351,7 @@ class Game(object):
         """
         pass
 
-    def event_paused_joystick_button_release(self, joystick, button):
+    def event_paused_joystick_button_release(self, name, ID, button):
         """Joystick button release event when paused.
 
         See the documentation for
@@ -1393,5 +1494,5 @@ class Game(object):
         new_num_channels = old_num_channels + 4
         pygame.mixer.set_num_channels(new_num_channels)
 
-        for i in xrange(old_num_channels, new_num_channels):
+        for i in range(old_num_channels, new_num_channels):
             self._available_channels.append(pygame.mixer.Channel(i))
