@@ -23,12 +23,12 @@ from __future__ import unicode_literals
 import sge
 
 
-__all__ = ['Room']
+__all__ = [u'Room']
 
 
 class Room(object):
 
-    """Class for rooms.
+    u"""Class for rooms.
 
     This class stores the settings and objects found in a room.  Rooms
     are used to create separate parts of the game, such as levels and
@@ -64,9 +64,18 @@ class Room(object):
 
     .. attribute:: objects
 
-       A tuple containing all :class:`sge.StellarClass` objects in the
+       A list containing all :class:`sge.StellarClass` objects in the
        room.  (Read-only)
 
+    .. attribute:: objects_by_class
+
+       A dictionary of lists containing all :class:`sge.StellarClass`
+       objects in the room, separated by class.  The dictionary keys are
+       classes that have been registered with
+       :meth:`sge.Game.register_class`, and the lists contain only
+       those objects which are instances of the class indicated by the
+       respective key.  (Read-only)
+
     .. attribute:: room_number
 
        The index of this room in the game, where ``0`` is the first
@@ -77,7 +86,7 @@ class Room(object):
 
     def __init__(self, objects=(), width=None, height=None, views=None,
                  background=None, background_x=0, background_y=0):
-        """Constructor method.
+        u"""Constructor method.
 
         Arguments:
 
@@ -117,7 +126,7 @@ class Room(object):
         if background is not None:
             self.background = background
         else:
-            self.background = sge.Background((), 'black')
+            self.background = sge.Background((), u'black')
         self._start_background = self.background
 
         self.room_number = len(sge.game.rooms)
@@ -126,11 +135,15 @@ class Room(object):
         self._started = False
         self._has_started = False
 
-        self.objects = ()
+        self.objects = []
+        self.objects_by_class = {}
+        for cls in sge.game.registered_classes:
+            self.objects_by_class[cls] = []
+
         self.add(sge.game.mouse)
         for obj in objects:
             self.add(obj)
-        self._start_objects = self.objects
+        self._start_objects = self.objects[:]
 
         self._object_start_x = {}
         self._object_start_y = {}
@@ -146,7 +159,7 @@ class Room(object):
         self._object_start_collision_precise = {}
 
     def add(self, obj):
-        """Add a StellarClass object to the room.
+        u"""Add a StellarClass object to the room.
 
         Arguments:
 
@@ -157,16 +170,19 @@ class Room(object):
             obj = sge.game.objects[obj]
 
         if obj not in self.objects:
-            new_objects = list(self.objects)
-            new_objects.append(obj)
-            self.objects = tuple(new_objects)
+            self.objects.append(obj)
+
+            for cls in self.objects_by_class:
+                if isinstance(obj, cls):
+                    self.objects_by_class[cls].append(obj)
+
             if self is sge.game.current_room:
                 sge.game._pygame_sprites.add(obj._pygame_sprite, layer=obj.z)
                 if self._started:
                     obj.event_create()
 
     def start(self):
-        """Start the room.
+        u"""Start the room.
 
         If the room has been changed, reset it to its original state.
 
@@ -177,7 +193,7 @@ class Room(object):
         self.resume()
 
     def resume(self):
-        """Continue the room from where it left off.
+        u"""Continue the room from where it left off.
 
         If the room is unchanged (e.g. has not been started yet), this
         method behaves in the same way that :meth:`sge.Room.start` does.
@@ -227,7 +243,7 @@ class Room(object):
         self._has_started = True
 
     def end(self, next_room=None, resume=True):
-        """End the current room.
+        u"""End the current room.
 
         Arguments:
 
@@ -259,7 +275,7 @@ class Room(object):
             sge.game.end()
 
     def project_dot(self, x, y, z, color):
-        """Project a single-pixel dot onto the room.
+        u"""Project a single-pixel dot onto the room.
 
         Arguments:
 
@@ -280,7 +296,7 @@ class Room(object):
 
     def project_line(self, x1, y1, x2, y2, z, color, thickness=1,
                      anti_alias=False):
-        """Project a line segment onto the room.
+        u"""Project a line segment onto the room.
 
         Arguments:
 
@@ -314,7 +330,7 @@ class Room(object):
 
     def project_rectangle(self, x, y, z, width, height, fill=None,
                           outline=None, outline_thickness=1):
-        """Project a rectangle onto the room.
+        u"""Project a rectangle onto the room.
 
         Arguments:
 
@@ -343,7 +359,7 @@ class Room(object):
 
     def project_ellipse(self, x, y, z, width, height, fill=None,
                         outline=None, outline_thickness=1, anti_alias=False):
-        """Project an ellipse onto the room.
+        u"""Project an ellipse onto the room.
 
         Arguments:
 
@@ -379,7 +395,7 @@ class Room(object):
 
     def project_circle(self, x, y, z, radius, fill=None, outline=None,
                        outline_thickness=1, anti_alias=False):
-        """Project a circle onto the room.
+        u"""Project a circle onto the room.
 
         Arguments:
 
@@ -404,7 +420,7 @@ class Room(object):
         self.add(p)
 
     def project_sprite(self, sprite, image, x, y, z, blend_mode=None):
-        """Project a sprite onto the room.
+        u"""Project a sprite onto the room.
 
         Arguments:
 
@@ -419,7 +435,7 @@ class Room(object):
 
         """
         if blend_mode & sge.BLEND_SCREEN:
-            w = "Screen blend mode not supported. Normal blending used instead."
+            w = u"Screen blend mode not supported. Normal blending used instead."
             warnings.warn(w)
 
         pygame_flags = {
@@ -441,9 +457,9 @@ class Room(object):
         self.add(p)
 
     def project_text(self, font, text, x, y, z, width=None, height=None,
-                    color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                    color=u"black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
                     anti_alias=True):
-        """Project text onto the room.
+        u"""Project text onto the room.
 
         Arguments:
 
@@ -472,7 +488,7 @@ class Room(object):
         self.add(p)
 
     def event_room_start(self):
-        """Room start event.
+        u"""Room start event.
 
         Called when the room starts.  It is always called after any game
         start events and before any object create events occurring at
@@ -482,7 +498,7 @@ class Room(object):
         pass
 
     def event_room_resume(self):
-        """Room resume event.
+        u"""Room resume event.
 
         Called when the room resumes without being reset to its original
         state (i.e. via :meth:`sge.Room.resume`).
@@ -493,7 +509,7 @@ class Room(object):
         self.event_room_start()
 
     def event_room_end(self):
-        """Room end event.
+        u"""Room end event.
 
         Called when the room ends.  It is always called before any game
         end events occurring at the same time.
@@ -502,7 +518,7 @@ class Room(object):
         pass
 
     def event_step(self, time_passed):
-        """Room step event.
+        u"""Room step event.
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
@@ -511,7 +527,7 @@ class Room(object):
         pass
 
     def event_key_press(self, key, char):
-        """Key press event.
+        u"""Key press event.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -520,7 +536,7 @@ class Room(object):
         pass
 
     def event_key_release(self, key):
-        """Key release event.
+        u"""Key release event.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -529,7 +545,7 @@ class Room(object):
         pass
 
     def event_mouse_move(self, x, y):
-        """Mouse move event.
+        u"""Mouse move event.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -538,7 +554,7 @@ class Room(object):
         pass
 
     def event_mouse_button_press(self, button):
-        """Mouse button press event.
+        u"""Mouse button press event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -547,7 +563,7 @@ class Room(object):
         pass
 
     def event_mouse_button_release(self, button):
-        """Mouse button release event.
+        u"""Mouse button release event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -556,8 +572,8 @@ class Room(object):
         """
         pass
 
-    def event_joystick_axis_move(self, joystick, axis, value):
-        """Joystick axis move event.
+    def event_joystick_axis_move(self, name, ID, axis, value):
+        u"""Joystick axis move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -565,8 +581,8 @@ class Room(object):
         """
         pass
 
-    def event_joystick_hat_move(self, joystick, hat, x, y):
-        """Joystick HAT move event.
+    def event_joystick_hat_move(self, name, ID, hat, x, y):
+        u"""Joystick HAT move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -574,8 +590,8 @@ class Room(object):
         """
         pass
 
-    def event_joystick_trackball_move(self, joystick, ball, x, y):
-        """Joystick trackball move event.
+    def event_joystick_trackball_move(self, name, ID, ball, x, y):
+        u"""Joystick trackball move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -584,8 +600,8 @@ class Room(object):
         """
         pass
 
-    def event_joystick_button_press(self, joystick, button):
-        """Joystick button press event.
+    def event_joystick_button_press(self, name, ID, button):
+        u"""Joystick button press event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -594,8 +610,8 @@ class Room(object):
         """
         pass
 
-    def event_joystick_button_release(self, joystick, button):
-        """Joystick button release event.
+    def event_joystick_button_release(self, name, ID, button):
+        u"""Joystick button release event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -605,7 +621,7 @@ class Room(object):
         pass
 
     def event_gain_keyboard_focus(self):
-        """Gain keyboard focus event.
+        u"""Gain keyboard focus event.
 
         See the documentation for
         :meth:`sge.Game.event_gain_keyboard_focus` for more information.
@@ -614,19 +630,19 @@ class Room(object):
         pass
 
     def event_lose_keyboard_focus(self):
-        """Lose keyboard focus event.
+        u"""Lose keyboard focus event.
 
         See the documentation for
         :meth:`sge.Game.event_lose_keyboard_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = "event_loose_keyboard_focus"
+            bad_name = u"event_loose_keyboard_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_gain_mouse_focus(self):
-        """Gain mouse focus event.
+        u"""Gain mouse focus event.
 
         See the documentation for
         :meth:`sge.Game.event_gain_mouse_focus` for more information.
@@ -635,19 +651,19 @@ class Room(object):
         pass
 
     def event_lose_mouse_focus(self):
-        """Lose mouse focus event.
+        u"""Lose mouse focus event.
 
         See the documentation for
         :meth:`sge.Game.event_lose_mouse_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = "event_loose_mouse_focus"
+            bad_name = u"event_loose_mouse_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_close(self):
-        """Close event.
+        u"""Close event.
 
         See the documentation for :meth:`sge.Game.event_close` for more
         information.  This is always called before any game close events
@@ -657,7 +673,7 @@ class Room(object):
         pass
 
     def event_paused_key_press(self, key, char):
-        """Key press event when paused.
+        u"""Key press event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -666,7 +682,7 @@ class Room(object):
         pass
 
     def event_paused_key_release(self, key):
-        """Key release event when paused.
+        u"""Key release event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -675,7 +691,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_move(self, x, y):
-        """Mouse move event when paused.
+        u"""Mouse move event when paused.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -684,7 +700,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_button_press(self, button):
-        """Mouse button press event when paused.
+        u"""Mouse button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -693,7 +709,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_button_release(self, button):
-        """Mouse button release event when paused.
+        u"""Mouse button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -702,8 +718,8 @@ class Room(object):
         """
         pass
 
-    def event_paused_joystick_axis_move(self, joystick, axis, value):
-        """Joystick axis move event when paused.
+    def event_paused_joystick_axis_move(self, name, ID, axis, value):
+        u"""Joystick axis move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -711,8 +727,8 @@ class Room(object):
         """
         pass
 
-    def event_paused_joystick_hat_move(self, joystick, hat, x, y):
-        """Joystick HAT move event when paused.
+    def event_paused_joystick_hat_move(self, name, ID, hat, x, y):
+        u"""Joystick HAT move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -720,8 +736,8 @@ class Room(object):
         """
         pass
 
-    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
-        """Joystick trackball move event when paused.
+    def event_paused_joystick_trackball_move(self, name, ID, ball, x, y):
+        u"""Joystick trackball move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -730,8 +746,8 @@ class Room(object):
         """
         pass
 
-    def event_paused_joystick_button_press(self, joystick, button):
-        """Joystick button press event when paused.
+    def event_paused_joystick_button_press(self, name, ID, button):
+        u"""Joystick button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -740,8 +756,8 @@ class Room(object):
         """
         pass
 
-    def event_paused_joystick_button_release(self, joystick, button):
-        """Joystick button release event when paused.
+    def event_paused_joystick_button_release(self, name, ID, button):
+        u"""Joystick button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -751,7 +767,7 @@ class Room(object):
         pass
 
     def event_paused_gain_keyboard_focus(self):
-        """Gain keyboard focus event when paused.
+        u"""Gain keyboard focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_gain_keyboard_focus` for more information.
@@ -760,19 +776,19 @@ class Room(object):
         pass
 
     def event_paused_lose_keyboard_focus(self):
-        """Lose keyboard focus event when paused.
+        u"""Lose keyboard focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_lose_keyboard_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = "event_paused_loose_keyboard_focus"
+            bad_name = u"event_paused_loose_keyboard_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_paused_gain_mouse_focus(self):
-        """Gain mouse focus event when paused.
+        u"""Gain mouse focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_gain_mouse_focus` for more information.
@@ -781,19 +797,19 @@ class Room(object):
         pass
 
     def event_paused_lose_mouse_focus(self):
-        """Lose mouse focus event when paused.
+        u"""Lose mouse focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_lose_mouse_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = "event_paused_loose_mouse_focus"
+            bad_name = u"event_paused_loose_mouse_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_paused_close(self):
-        """Close event when paused.
+        u"""Close event when paused.
 
         See the documentation for :meth:`sge.Game.event_close` for more
         information.
@@ -821,7 +837,7 @@ class Room(object):
         self.height = self._start_height
         self.views = self._start_views
         self.background = self._start_background
-        self.objects = self._start_objects
+        self.objects = self._start_objects[:]
 
         for view in self.views:
             view._reset()
