@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -28,12 +28,12 @@ import pygame
 import sge
 
 
-__all__ = ['Sprite']
+__all__ = [u'Sprite']
 
 
 class Sprite(object):
 
-    """Class which holds information for images and animations.
+    u"""Class which holds information for images and animations.
 
     This class stores images and information about how the SGE is to use
     those images.
@@ -47,10 +47,28 @@ class Sprite(object):
 
        The width of the sprite.
 
+       .. note::
+
+          Changing this attribute is a destructive transformation: it
+          can result in loss of pixel information, especially if it is
+          done repeatedly.  Because of this, it is advised that you do
+          not adjust this value for routine scaling.  Use the
+          :attr:`image_xscale` attribute of a :class:`sge.StellarClass`
+          object instead.
+
     .. attribute:: height
 
        The height of the sprite.
 
+       .. note::
+
+          Changing this attribute is a destructive transformation: it
+          can result in loss of pixel information, especially if it is
+          done repeatedly.  Because of this, it is advised that you do
+          not adjust this value for routine scaling.  Use the
+          :attr:`image_yscale` attribute of a :class:`sge.StellarClass`
+          object instead.
+
     .. attribute:: origin_x
 
        The horizontal location of the origin relative to the left edge
@@ -72,6 +90,11 @@ class Sprite(object):
 
        The suggested rate in frames per second to animate the image at.
 
+    .. attribute:: speed
+
+       The suggested rate to animate the image at as a factor of
+       :attr:`sge.game.fps`.
+
     .. attribute:: bbox_x
 
        The horizontal location relative to the sprite of the suggested
@@ -136,6 +159,14 @@ class Sprite(object):
             self._transparent = value
             self._refresh()
 
+    @property
+    def speed(self):
+        return self.fps / sge.game.fps
+
+    @speed.setter
+    def speed(self, value):
+        self.fps = value * sge.game.fps
+
     @property
     def bbox_x(self):
         return self._bbox_x
@@ -162,7 +193,7 @@ class Sprite(object):
                  origin_x=0, origin_y=0, transparent=True, fps=60,
                  bbox_x=None, bbox_y=None, bbox_width=None,
                  bbox_height=None):
-        """Constructor method.
+        u"""Constructor method.
 
         Arguments:
 
@@ -203,7 +234,7 @@ class Sprite(object):
 
         """
         if sge.DEBUG:
-            print('Creating sprite "{0}"'.format(name))
+            print(u'Creating sprite "{0}"'.format(name))
 
         self.name = name
         self._transparent = None
@@ -224,7 +255,7 @@ class Sprite(object):
                 i = 0
                 while self.id in sge.game.sprites:
                     i += 1
-                    self.id = "{0}{1}".format(self.name, i)
+                    self.id = u"{0}{1}".format(self.name, i)
 
             for path in sge.image_directories:
                 if os.path.isdir(path):
@@ -234,11 +265,11 @@ class Sprite(object):
                         if fname.startswith(name) and os.path.isfile(full_fname):
                             root, ext = os.path.splitext(fname)
                             if root == name:
-                                split = [name, '']
-                            elif root.rsplit('-', 1)[0] == name:
-                                split = root.rsplit('-', 1)
-                            elif root.rsplit('_', 1)[0] == name:
-                                split = root.rsplit('_', 1)
+                                split = [name, u'']
+                            elif root.rsplit(u'-', 1)[0] == name:
+                                split = root.rsplit(u'-', 1)
+                            elif root.rsplit(u'_', 1)[0] == name:
+                                split = root.rsplit(u'_', 1)
                             else:
                                 continue
 
@@ -249,7 +280,7 @@ class Sprite(object):
                                 while len(fname_frames) - 1 < n:
                                     fname_frames.append(None)
                                 fname_frames[n] = full_fname
-                            elif (split[1].startswith('strip') and
+                            elif (split[1].startswith(u'strip') and
                                   split[1][5:].isdigit()):
                                 fname_strip = full_fname
 
@@ -260,7 +291,7 @@ class Sprite(object):
                     self._baseimages.append(img)
                 except pygame.error:
                     if sge.DEBUG:
-                        print("Ignored {0}; not a valid image.".format(fname_single))
+                        print(u"Ignored {0}; not a valid image.".format(fname_single))
 
             if not self._baseimages and any(fname_frames):
                 # Load the multiple images
@@ -270,18 +301,18 @@ class Sprite(object):
                             self._baseimages.append(pygame.image.load(fname))
                         except pygame.error:
                             if sge.DEBUG:
-                                print("Ignored {0}; not a valid image.".format(fname))
+                                print(u"Ignored {0}; not a valid image.".format(fname))
 
             if not self._baseimages and fname_strip:
                 # Load the strip (sprite sheet)
                 root, ext = os.path.splitext(os.path.basename(fname_strip))
-                assert '-' in root or '_' in root
-                assert (root.rsplit('-', 1)[0] == name or
-                        root.rsplit('_', 1)[0] == name)
-                if root.rsplit('-', 1)[0] == name:
-                    split = root.rsplit('-', 1)
+                assert u'-' in root or u'_' in root
+                assert (root.rsplit(u'-', 1)[0] == name or
+                        root.rsplit(u'_', 1)[0] == name)
+                if root.rsplit(u'-', 1)[0] == name:
+                    split = root.rsplit(u'-', 1)
                 else:
-                    split = root.rsplit('_', 1)
+                    split = root.rsplit(u'_', 1)
 
                 try:
                     sheet = pygame.image.load(fname_strip)
@@ -296,13 +327,13 @@ class Sprite(object):
                         self._baseimages.append(img)
                 except pygame.error:
                     if sge.DEBUG:
-                        print("Ignored {0}; not a valid image.".format(fname_strip))
+                        print(u"Ignored {0}; not a valid image.".format(fname_strip))
 
             if not self._baseimages:
-                print("Directories searched:")
+                print(u"Directories searched:")
                 for d in sge.image_directories:
                     print(os.path.normpath(os.path.abspath(d)))
-                msg = 'Files for sprite name "{0}" not found.'.format(name)
+                msg = u'Files for sprite name "{0}" not found.'.format(name)
                 raise IOError(msg)
         else:
             # Name is None; default to a blank rectangle.
@@ -312,7 +343,7 @@ class Sprite(object):
                 height = 32
 
             # Choose name
-            self.name = "sge-pygame-dynamicsprite"
+            self.name = u"sge-pygame-dynamicsprite"
             if ID is not None:
                 self.id = ID
             else:
@@ -324,7 +355,7 @@ class Sprite(object):
             img.fill(pygame.Color(0, 0, 0, 0))
             self._baseimages.append(img)
             if sge.DEBUG:
-                print("renamed to {0}, ID is {1}".format(self.name, self.id))
+                print(u"renamed to {0}, ID is {1}".format(self.name, self.id))
 
         if width is None:
             width = 1
@@ -357,7 +388,7 @@ class Sprite(object):
         sge.game.sprites[self.id] = self
 
     def draw_dot(self, x, y, color, frame=None):
-        """Draw a single-pixel dot on the sprite.
+        u"""Draw a single-pixel dot on the sprite.
 
         Arguments:
 
@@ -380,7 +411,7 @@ class Sprite(object):
 
     def draw_line(self, x1, y1, x2, y2, color, thickness=1, anti_alias=False,
                   frame=None):
-        """Draw a line segment on the sprite.
+        u"""Draw a line segment on the sprite.
 
         Arguments:
 
@@ -416,7 +447,7 @@ class Sprite(object):
 
     def draw_rectangle(self, x, y, width, height, fill=None, outline=None,
                        outline_thickness=1, frame=None):
-        """Draw a rectangle on the sprite.
+        u"""Draw a rectangle on the sprite.
 
         Arguments:
 
@@ -460,7 +491,7 @@ class Sprite(object):
 
     def draw_ellipse(self, x, y, width, height, fill=None, outline=None,
                      outline_thickness=1, anti_alias=False, frame=None):
-        """Draw an ellipse on the sprite.
+        u"""Draw an ellipse on the sprite.
 
         Arguments:
 
@@ -505,7 +536,7 @@ class Sprite(object):
 
     def draw_circle(self, x, y, radius, fill=None, outline=None,
                     outline_thickness=1, anti_alias=False, frame=None):
-        """Draw a circle on the sprite.
+        u"""Draw a circle on the sprite.
 
         Arguments:
 
@@ -546,7 +577,7 @@ class Sprite(object):
         self._refresh()
 
     def draw_sprite(self, sprite, image, x, y, frame=None, blend_mode=None):
-        """Draw another sprite on the sprite.
+        u"""Draw another sprite on the sprite.
 
         Arguments:
 
@@ -588,7 +619,7 @@ class Sprite(object):
         image %= len(sprite._baseimages)
 
         if blend_mode & sge.BLEND_SCREEN:
-            w = "Screen blend mode not supported. Normal blending used instead."
+            w = u"Screen blend mode not supported. Normal blending used instead."
             warnings.warn(w)
 
         pygame_flags = {
@@ -612,9 +643,9 @@ class Sprite(object):
         self._refresh()
 
     def draw_text(self, font, text, x, y, width=None, height=None,
-                  color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                  color=u"black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
                   anti_alias=True, frame=None):
-        """Draw text on the sprite.
+        u"""Draw text on the sprite.
 
         Arguments:
 
@@ -735,7 +766,7 @@ class Sprite(object):
         self._refresh()
 
     def draw_clear(self, frame=None):
-        """Erase everything from the sprite.
+        u"""Erase everything from the sprite.
 
         Arguments:
 
@@ -755,7 +786,7 @@ class Sprite(object):
         self._refresh()
 
     def save(self, fname):
-        """Save the sprite to an image file.
+        u"""Save the sprite to an image file.
 
         Arguments:
 
@@ -781,13 +812,13 @@ class Sprite(object):
         try:
             pygame.image.save(reel, fname)
         except pygame.error:
-            m = 'Couldn\'t save to "{0}"'.format(
+            m = u'Couldn\'t save to "{0}"'.format(
                 os.path.normpath(os.path.realpath(fname)))
             raise IOError(m)
 
     @classmethod
     def from_screenshot(cls, x=0, y=0, width=None, height=None):
-        """Return the current display on the screen as a sprite.
+        u"""Return the current display on the screen as a sprite.
 
         Arguments:
 
