@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -31,12 +31,12 @@ import pygame
 import sge
 
 
-__all__ = ['StellarClass', 'Mouse']
+__all__ = [u'StellarClass', u'Mouse']
 
 
 class StellarClass(object):
 
-    """Class for game objects.
+    u"""Class for game objects.
 
     This class is used for game objects, such as the player, enemies,
     bullets, and the HUD.  Generally, each type of object has its own
@@ -190,6 +190,11 @@ class StellarClass(object):
        The animation rate in frames per second.  If set to
        :const:`None`, the value recommended by the sprite is used.
 
+    .. attribute:: image_speed
+
+       The animation rate as a factor of :attr:`sge.game.fps`.  If set
+       to :const:`None`, the value recommended by the sprite is used.
+
     .. attribute:: image_xscale
 
        The horizontal scale factor for the sprite.
@@ -230,11 +235,13 @@ class StellarClass(object):
 
     .. attribute:: xprevious
 
-       The previous value of :attr:`x`.  (Read-only)
+       The value of :attr:`x` at the end of the previous frame.
+       (Read-only)
 
     .. attribute:: yprevious
 
-       The previous value of :attr:`y`.  (Read-only)
+       The value of :attr:`y` at the end of the previous frame.
+       (Read-only)
 
     """
 
@@ -244,7 +251,6 @@ class StellarClass(object):
 
     @x.setter
     def x(self, value):
-        self.xprevious = self._x
         self._x = value
 
         # Cause the Pygame sprite to make itself dirty
@@ -256,7 +262,6 @@ class StellarClass(object):
 
     @y.setter
     def y(self, value):
-        self.yprevious = self._y
         self._y = value
 
         # Cause the Pygame sprite to make itself dirty
@@ -276,6 +281,11 @@ class StellarClass(object):
         # Cause the Pygame sprite to make itself dirty
         self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
 
+        # Make the mouse have a higher Z-axis value
+        if (u"mouse" in sge.game.objects and
+                sge.game.objects[u"mouse"].z <= value):
+            sge.game.objects[u"mouse"]._z = value + 1
+
     @property
     def sprite(self):
         return self._sprite
@@ -429,10 +439,21 @@ class StellarClass(object):
                 # This would be caused by a round-off to 0 resulting
                 # from a much too high frame rate.  It would cause a
                 # division by 0 later, so this is meant to prevent that.
-                self._frame_time = 0.01
+                self._frame_time = 0.000001
         else:
             self._frame_time = None
 
+    @property
+    def image_speed(self):
+        return self.image_fps / sge.game.fps
+
+    @image_speed.setter
+    def image_speed(self, value):
+        if value is None:
+            value = self.sprite.speed if self.sprite is not None else 0
+
+        self.image_fps = value * sge.game.fps
+
     def __init__(self, x, y, z=0, ID=None, sprite=None, visible=True,
                  active=True, detects_collisions=True, bbox_x=None,
                  bbox_y=None, bbox_width=None, bbox_height=None,
@@ -440,7 +461,7 @@ class StellarClass(object):
                  xvelocity=0, yvelocity=0, image_index=0, image_fps=None,
                  image_xscale=1, image_yscale=1, image_rotation=0,
                  image_alpha=255, image_blend=None):
-        """Constructor method.
+        u"""Constructor method.
 
         Arguments:
 
@@ -515,7 +536,10 @@ class StellarClass(object):
 
         self._rect = pygame.Rect(self.bbox_x, self.bbox_y, self.bbox_width,
                                  self.bbox_height)
-        self._pygame_sprite = _PygameSprite(self)
+        if self.id != u"mouse":
+            self._pygame_sprite = _PygameSprite(self)
+        else:
+            self._pygame_sprite = _FakePygameSprite(self)
         self._set_mask()
 
         self.z = z
@@ -534,7 +558,7 @@ class StellarClass(object):
         self._start_collision_precise = self.collision_precise
 
     def collides(self, other, x=None, y=None):
-        """Return whether or not this object collides with another.
+        u"""Return whether or not this object collides with another.
 
         Arguments:
 
@@ -551,11 +575,16 @@ class StellarClass(object):
         """
         if isinstance(other, StellarClass):
             others = [other]
+        elif other in sge.game.current_room.objects_by_class:
+            others = []
+            for obj in sge.game.current_room.objects_by_class[other]:
+                if obj.detects_collisions:
+                    others.append(obj)
         else:
             others = []
             for ref in sge.game._colliders:
-                if isinstance(obj(), other):
-                    others.append(obj())
+                if isinstance(ref(), other):
+                    others.append(ref())
 
         # Change x and y to be offset values; these are easier to use.
         if x is not None:
@@ -609,18 +638,19 @@ class StellarClass(object):
                             other._hitmask[
                                 a + other_xoffset][b + other_yoffset]):
                             return True
-
-                return False
                         
             else:
                 # Use bounding boxes.
-                return (self.bbox_left + x < other.bbox_right and
+                if (self.bbox_left + x < other.bbox_right and
                         self.bbox_right + x > other.bbox_left and
                         self.bbox_top + y < other.bbox_bottom and
-                        self.bbox_bottom + y > other.bbox_top)
+                        self.bbox_bottom + y > other.bbox_top):
+                    return True
+
+        return False
 
     def set_alarm(self, alarm_id, value):
-        """Set an alarm.
+        u"""Set an alarm.
 
         Arguments:
 
@@ -641,7 +671,7 @@ class StellarClass(object):
             del self._alarms[alarm_id]
 
     def get_alarm(self, alarm_id):
-        """Return the value of an alarm.
+        u"""Return the value of an alarm.
 
         Arguments:
 
@@ -656,7 +686,7 @@ class StellarClass(object):
             return None
 
     def destroy(self):
-        """Destroy the object."""
+        u"""Destroy the object."""
         self.event_destroy()
         self._pygame_sprite.kill()
         del sge.game.objects[self.id]
@@ -666,10 +696,18 @@ class StellarClass(object):
             for obj in room.objects:
                 if obj is not self:
                     new_objects.append(obj)
-            room.objects = tuple(new_objects)
+            room.objects = new_objects
+
+            for cls in room.objects_by_class:
+                if isinstance(self, cls):
+                    new_objects = []
+                    for obj in room.objects_by_class[cls]:
+                        if obj is not self:
+                            new_objects.append(obj)
+                    room.objects_by_class[cls] = new_objects
 
     def event_create(self):
-        """Create event.
+        u"""Create event.
 
         Called when the object is created.  It is always called after
         any room start events occurring at the same time.
@@ -678,11 +716,47 @@ class StellarClass(object):
         pass
 
     def event_destroy(self):
-        """Destroy event."""
+        u"""Destroy event."""
+        pass
+
+    def event_begin_step(self, time_passed):
+        u"""Begin step event.
+
+        This event is executed each frame before automatic updates to
+        objects (such as the effects of the speed variables).
+
+        See the documentation for :meth:`sge.Game.event_step` for more
+        information.
+
+        .. note::
+
+           Automatic updates, the only occurances between this event and
+           :meth:`sge.StellarClass.event_step`, do not occur, so there
+           is no need for an "inactive" variant of this event.  Use
+           :meth:`sge.StellarClass.event_inactive_step` instead.
+
+        """
         pass
 
     def event_step(self, time_passed):
-        """Step event.
+        u"""Step event.
+
+        This event is executed each frame after automatic updates to
+        objects (such as the effects of the speed variables), but before
+        collision events.
+
+        See the documentation for :meth:`sge.Game.event_step` for more
+        information.
+
+        """
+        pass
+
+    def event_end_step(self, time_passed):
+        u"""Step event.
+
+        This event is executed each frame after automatic updates to
+        objects (such as the effects of the speed variables), but before
+        collision events.
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
@@ -691,7 +765,7 @@ class StellarClass(object):
         pass
 
     def event_alarm(self, alarm_id):
-        """Alarm event.
+        u"""Alarm event.
 
         Called when the value of an alarm reaches 0.
 
@@ -704,7 +778,7 @@ class StellarClass(object):
         pass
 
     def event_animation_end(self):
-        """Animation End event.
+        u"""Animation End event.
 
         Called when an animation cycle ends.
 
@@ -712,7 +786,7 @@ class StellarClass(object):
         pass
 
     def event_key_press(self, key, char):
-        """Key press event.
+        u"""Key press event.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -721,7 +795,7 @@ class StellarClass(object):
         pass
 
     def event_key_release(self, key):
-        """Key release event.
+        u"""Key release event.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -730,7 +804,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_move(self, x, y):
-        """Mouse move event.
+        u"""Mouse move event.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -739,7 +813,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_button_press(self, button):
-        """Mouse button press event.
+        u"""Mouse button press event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -748,7 +822,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_button_release(self, button):
-        """Mouse button release event.
+        u"""Mouse button release event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -757,8 +831,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_joystick_axis_move(self, joystick, axis, value):
-        """Joystick axis move event.
+    def event_joystick_axis_move(self, name, ID, axis, value):
+        u"""Joystick axis move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -766,8 +840,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_joystick_hat_move(self, joystick, hat, x, y):
-        """Joystick HAT move event.
+    def event_joystick_hat_move(self, name, ID, hat, x, y):
+        u"""Joystick HAT move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -775,8 +849,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_joystick_trackball_move(self, joystick, ball, x, y):
-        """Joystick trackball move event.
+    def event_joystick_trackball_move(self, name, ID, ball, x, y):
+        u"""Joystick trackball move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -785,8 +859,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_joystick_button_press(self, joystick, button):
-        """Joystick button press event.
+    def event_joystick_button_press(self, name, ID, button):
+        u"""Joystick button press event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -795,8 +869,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_joystick_button_release(self, joystick, button):
-        """Joystick button release event.
+    def event_joystick_button_release(self, name, ID, button):
+        u"""Joystick button release event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -806,7 +880,7 @@ class StellarClass(object):
         pass
 
     def event_collision(self, other):
-        """Default collision event.
+        u"""Default collision event.
 
         Called when another object collides with this object and none of
         the directional collision events are appropriate.  In
@@ -822,7 +896,7 @@ class StellarClass(object):
         pass
 
     def event_collision_left(self, other):
-        """Left collision event.
+        u"""Left collision event.
 
         Called when another object collides with this object's left
         side.
@@ -838,7 +912,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_right(self, other):
-        """Right collision event.
+        u"""Right collision event.
 
         Called when another object collides with this object's right
         side.
@@ -854,7 +928,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_top(self, other):
-        """Top collision event.
+        u"""Top collision event.
 
         Called when another object collides with this object's top side.
 
@@ -869,7 +943,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_bottom(self, other):
-        """Bottom collision event.
+        u"""Bottom collision event.
 
         Called when another object collides with this object's bottom
         side.
@@ -885,16 +959,28 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_inactive_step(self, time_passed):
-        """Step event when this object is inactive.
+        u"""Step event when this object is inactive.
 
-        See the documentation for :meth:`sge.Game.event_step` for more
-        information.  The object is considered to be inactive when
-        :attr:`active` is :attr:`False`.
+        See the documentation for :meth:`sge.StellarClass.event_step`
+        for more information.  The object is considered to be inactive
+        when :attr:`active` is :attr:`False`.
 
         """
+        pass
+
+    def event_inactive_end_step(self, time_passed):
+        u"""End step event when this object is inactive.
+
+        See the documentation for
+        :meth:`sge.StellarClass.event_end_step` for more information.
+        The object is considered to be inactive when :attr:`active` is
+        :attr:`False`.
+
+        """
+        pass
 
     def event_inactive_key_press(self, key, char):
-        """Key press event when this object is inactive.
+        u"""Key press event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.  The object is considered to be inactive when
@@ -904,7 +990,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_key_release(self, key):
-        """Key release event when this object is inactive.
+        u"""Key release event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.  The object is considered to be inactive when
@@ -914,7 +1000,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_move(self, x, y):
-        """Mouse move event when this object is inactive.
+        u"""Mouse move event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.  The object is considered to be inactive when
@@ -924,7 +1010,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_button_press(self, button):
-        """Mouse button press event when this object is inactive.
+        u"""Mouse button press event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -935,7 +1021,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_button_release(self, button):
-        """Mouse button release event when this object is inactive.
+        u"""Mouse button release event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -945,8 +1031,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_inactive_joystick_axis_move(self, joystick, axis, value):
-        """Joystick axis move event when this object is inactive.
+    def event_inactive_joystick_axis_move(self, name, ID, axis, value):
+        u"""Joystick axis move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -956,8 +1042,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_inactive_joystick_hat_move(self, joystick, hat, x, y):
-        """Joystick HAT move event when this object is inactive.
+    def event_inactive_joystick_hat_move(self, name, ID, hat, x, y):
+        u"""Joystick HAT move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -967,8 +1053,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_inactive_joystick_trackball_move(self, joystick, ball, x, y):
-        """Joystick trackball move event when this object is inactive.
+    def event_inactive_joystick_trackball_move(self, name, ID, ball, x, y):
+        u"""Joystick trackball move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -978,8 +1064,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_inactive_joystick_button_press(self, joystick, button):
-        """Joystick button press event when this object is inactive.
+    def event_inactive_joystick_button_press(self, name, ID, button):
+        u"""Joystick button press event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -989,8 +1075,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_inactive_joystick_button_release(self, joystick, button):
-        """Joystick button release event when this object is inactive.
+    def event_inactive_joystick_button_release(self, name, ID, button):
+        u"""Joystick button release event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -1001,7 +1087,7 @@ class StellarClass(object):
         pass
 
     def event_paused_key_press(self, key, char):
-        """Key press event when paused.
+        u"""Key press event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -1010,7 +1096,7 @@ class StellarClass(object):
         pass
 
     def event_paused_key_release(self, key):
-        """Key release event when paused.
+        u"""Key release event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -1019,7 +1105,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_move(self, x, y):
-        """Mouse move event when paused.
+        u"""Mouse move event when paused.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -1028,7 +1114,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_button_press(self, button):
-        """Mouse button press event when paused.
+        u"""Mouse button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -1037,7 +1123,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_button_release(self, button):
-        """Mouse button release event when paused.
+        u"""Mouse button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -1046,8 +1132,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_paused_joystick_axis_move(self, joystick, axis, value):
-        """Joystick axis move event when paused.
+    def event_paused_joystick_axis_move(self, name, ID, axis, value):
+        u"""Joystick axis move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -1055,8 +1141,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_paused_joystick_hat_move(self, joystick, hat, x, y):
-        """Joystick HAT move event when paused.
+    def event_paused_joystick_hat_move(self, name, ID, hat, x, y):
+        u"""Joystick HAT move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -1064,8 +1150,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_paused_joystick_trackball_move(self, joystick, ball, x, y):
-        """Joystick trackball move event when paused.
+    def event_paused_joystick_trackball_move(self, name, ID, ball, x, y):
+        u"""Joystick trackball move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -1074,8 +1160,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_paused_joystick_button_press(self, joystick, button):
-        """Joystick button press event when paused.
+    def event_paused_joystick_button_press(self, name, ID, button):
+        u"""Joystick button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -1084,8 +1170,8 @@ class StellarClass(object):
         """
         pass
 
-    def event_paused_joystick_button_release(self, joystick, button):
-        """Joystick button release event when paused.
+    def event_paused_joystick_button_release(self, name, ID, button):
+        u"""Joystick button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -1096,7 +1182,7 @@ class StellarClass(object):
 
     @classmethod
     def create(cls, *args, **kwargs):
-        """Create an object of this class in the current room it.
+        u"""Create an object of this class in the current room.
 
         ``args`` and ``kwargs`` are passed to the constructor method of
         ``cls`` as arguments.  Calling
@@ -1119,7 +1205,7 @@ class StellarClass(object):
             self._anim_count %= self._frame_time
 
         # Alarms
-        for a in self._alarms.keys():
+        for a in self._alarms:
             self._alarms[a] -= delta_mult
             if self._alarms[a] <= 0:
                 del self._alarms[a]
@@ -1127,7 +1213,7 @@ class StellarClass(object):
 
         # Movement
         if self.xvelocity or self.yvelocity:
-            if self.id != "mouse":
+            if self.id != u"mouse":
                 self.x += self.xvelocity * delta_mult
                 self.y += self.yvelocity * delta_mult
 
@@ -1148,84 +1234,44 @@ class StellarClass(object):
                     break
 
             if self.collides(other):
-                other.x = other.xprevious
-                other.y = other.yprevious
-                collision_new = not self.collides(other, self.xprevious,
-                                                  self.yprevious)
-                if collision_new:
-                    xcollision = not self.collides(other, x=self.xprevious)
-                    ycollision = not self.collides(other, y=self.yprevious)
+                self_prev_bbox_left = self.xprevious + self.bbox_x
+                self_prev_bbox_right = self_prev_bbox_left + self.bbox_width
+                self_prev_bbox_top = self.yprevious + self.bbox_y
+                self_prev_bbox_bottom = self_prev_bbox_top + self.bbox_height
+                other_prev_bbox_left = other.xprevious + other.bbox_x
+                other_prev_bbox_right = other_prev_bbox_left + other.bbox_width
+                other_prev_bbox_top = other.yprevious + other.bbox_y
+                other_prev_bbox_bottom = other_prev_bbox_top + other.bbox_height
+
+                if self_prev_bbox_right <= other_prev_bbox_left:
+                    xdirection = 1
+                elif self_prev_bbox_left >= other_prev_bbox_right:
+                    xdirection = -1
                 else:
-                    xcollision = False
-                    ycollision = False
+                    xdirection = 0
 
-                other.x = other.xprevious
-                other.y = other.yprevious
+                if self_prev_bbox_bottom <= other_prev_bbox_top:
+                    ydirection = 1
+                elif self_prev_bbox_top >= other_prev_bbox_bottom:
+                    ydirection = -1
+                else:
+                    ydirection = 0
 
-                if xcollision and ycollision:
-                    # Corner collision; determine direction by distance.
-                    if self.xvelocity > 0:
-                        xdepth = (self.bbox_right - other.bbox_left)
-                    else:
-                        xdepth = (other.bbox_right - self.bbox_left)
-
-                    if self.yvelocity > 0:
-                        ydepth = (self.bbox_bottom - other.bbox_top)
-                    else:
-                        ydepth = (other.bbox_bottom - self.bbox_top)
-
-                    if xdepth > ydepth:
-                        if self.xvelocity > 0:
-                            self.event_collision_right(other)
-                            other.event_collision_left(self)
-                        else:
-                            self.event_collision_left(other)
-                            other.event_collision_right(self)
-                    else:
-                        if self.yvelocity > 0:
-                            self.event_collision_bottom(other)
-                            other.event_collision_top(self)
-                        else:
-                            self.event_collision_top(other)
-                            other.event_collision_bottom(self)
-
-                elif xcollision:
-                    # Horizontal collision only.
-                    if self.xvelocity > 0:
+                if xdirection or ydirection:
+                    if xdirection == 1:
                         self.event_collision_right(other)
                         other.event_collision_left(self)
-                    else:
+                    elif xdirection == -1:
                         self.event_collision_left(other)
                         other.event_collision_right(self)
 
-                elif ycollision:
-                    # Vertical collision only.
-                    if self.yvelocity > 0:
+                    if ydirection == 1:
                         self.event_collision_bottom(other)
                         other.event_collision_top(self)
-                    else:
+                    elif ydirection == -1:
                         self.event_collision_top(other)
                         other.event_collision_bottom(self)
-
-                elif collision_new:
-                    # Wedge collision (both vertical and horizontal
-                    # collisions).
-                    if self.xvelocity > 0:
-                        self.event_collision_right(other)
-                        other.event_collision_left(self)
-                    else:
-                        self.event_collision_left(other)
-                        other.event_collision_right(self)
-                    if self.yvelocity > 0:
-                        self.event_collision_bottom(other)
-                        other.event_collision_top(self)
-                    else:
-                        self.event_collision_top(other)
-                        other.event_collision_bottom(self)
-
                 else:
-                    # No directional collision; this is
-                    # a continuous collision.
                     self.event_collision(other)
                     other.event_collision(self)
 
@@ -1310,7 +1356,7 @@ class Mouse(StellarClass):
                         view.yport <= mouse_y <= view.yport + view.height):
                     # We save this value so that if the mouse is in none of
                     # the views, the last known position in a view is used.
-                    self._x = mouse_x - view.x
+                    self._x = (mouse_x - view.xport) + view.x
                     break
 
             return self._x
@@ -1342,7 +1388,7 @@ class Mouse(StellarClass):
                         view.yport <= mouse_y <= view.yport + view.height):
                     # We save this value so that if the mouse is in none of
                     # the views, the last known position in a view is used.
-                    self._y = mouse_y - view.y
+                    self._y = (mouse_y - view.yport) + view.y
                     break
 
             return self._y
@@ -1384,9 +1430,7 @@ class Mouse(StellarClass):
     @visible.setter
     def visible(self, value):
         if sge.DEBUG:
-            print("mouse.visible has been set to:", value)
-            if value and hasattr(self, "_visible") and not self._visible:
-                raise ValueError
+            print(u"mouse.visible has been set to:", value)
 
         self._visible = value
         self.set_cursor()
@@ -1398,7 +1442,7 @@ class Mouse(StellarClass):
         self.mouse_yprevious = self.mouse_y
         self.previous_speeds = []
 
-        super(Mouse, self).__init__(0, 0, 0, ID='mouse')
+        super(Mouse, self).__init__(0, 0, 0, ID=u'mouse')
 
     def event_collision(self, other):
         sge.game.event_mouse_collision(other)
@@ -1638,6 +1682,17 @@ class _PygameSprite(pygame.sprite.DirtySprite):
                 self.image.set_colorkey((0, 0, 0))
 
 
+class _FakePygameSprite(_PygameSprite):
+
+    # Fake Pygame sprite for "use" by the mouse.
+
+    def update(self):
+        pass
+
+    def update_rect(self):
+        pass
+
+
 class _PygameOneTimeSprite(pygame.sprite.DirtySprite):
 
     # A regular DirtySprite that only displays once, and then destroys
